@@ -43,8 +43,8 @@ extern "C" {
 
 //////////////////////////////////////////////
 
-#define STOPPING_TIME (5)
-#define STARTING_TCP_RELAY_TIME (10)
+#define STOPPING_TIME (10)
+#define STARTING_TCP_RELAY_TIME (30)
 
 extern int clmessage_length;
 extern int do_not_use_channel;
@@ -99,7 +99,7 @@ void start_mclient(const char *remote_address, int port,
 		   int messagenumber, int mclient);
 
 int send_buffer(app_ur_conn_info *clnet_info, stun_buffer* message, int data_connection, app_tcp_conn_info *atc);
-int recv_buffer(app_ur_conn_info *clnet_info, stun_buffer* message, int sync, app_tcp_conn_info *atc);
+int recv_buffer(app_ur_conn_info *clnet_info, stun_buffer* message, int sync, int data_connection, app_tcp_conn_info *atc, stun_buffer* request_message);
 
 void client_input_handler(evutil_socket_t fd, short what, void* arg);
 
