@@ -96,6 +96,8 @@ typedef enum _SOCKET_TYPE SOCKET_TYPE;
 enum _SOCKET_APP_TYPE {
 	UNKNOWN_APP_SOCKET,
 	CLIENT_SOCKET,
+	HTTP_CLIENT_SOCKET,
+	HTTPS_CLIENT_SOCKET,
 	RELAY_SOCKET,
 	RELAY_RTCP_SOCKET,
 	TCP_CLIENT_DATA_SOCKET,
@@ -137,9 +139,16 @@ typedef struct _realm_options_t realm_options_t;
 
 //////// IP White/black listing ///////////
 
+struct _ip_range {
+	char str[257];
+	char realm[513];
+	ioa_addr_range enc;
+};
+
+typedef struct _ip_range ip_range_t;
+
 struct _ip_range_list {
-	char **ranges;
-	ioa_addr_range **encaddrsranges;
+	ip_range_t *rs;
 	size_t ranges_number;
 };
 
@@ -215,9 +224,11 @@ int create_relay_ioa_sockets(ioa_engine_handle e, ioa_socket_handle client_s,
 
 ioa_socket_handle  ioa_create_connecting_tcp_relay_socket(ioa_socket_handle s, ioa_addr *peer_addr, connect_cb cb, void *arg);
 
-int get_ioa_socket_from_reservation(ioa_engine_handle e, u64bits in_reservation_token, ioa_socket_handle *s);
+int get_ioa_socket_from_reservation(ioa_engine_handle e, u64bits in_reservation_token, ioa_socket_handle *s, u08bits *realm);
 
 int get_ioa_socket_address_family(ioa_socket_handle s);
+const char* get_ioa_socket_cipher(ioa_socket_handle s);
+const char* get_ioa_socket_ssl_method(ioa_socket_handle s);
 SOCKET_TYPE get_ioa_socket_type(ioa_socket_handle s);
 SOCKET_APP_TYPE get_ioa_socket_app_type(ioa_socket_handle s);
 const char* get_ioa_socket_tls_method(ioa_socket_handle s);
@@ -235,7 +246,7 @@ int register_callback_on_ioa_socket(ioa_engine_handle e, ioa_socket_handle s, in
 int send_data_from_ioa_socket_nbh(ioa_socket_handle s, ioa_addr* dest_addr, ioa_network_buffer_handle nbh, int ttl, int tos);
 void close_ioa_socket(ioa_socket_handle s);
 #define IOA_CLOSE_SOCKET(S) do { if(S) { close_ioa_socket(S); S = NULL; } } while(0)
-ioa_socket_handle detach_ioa_socket(ioa_socket_handle s, int full_detach);
+ioa_socket_handle detach_ioa_socket(ioa_socket_handle s);
 void detach_socket_net_data(ioa_socket_handle s);
 int set_df_on_ioa_socket(ioa_socket_handle s, int value);
 void set_do_not_use_df(ioa_socket_handle s);
@@ -265,6 +276,11 @@ void get_realm_options_by_name(char *realm, realm_options_t* ro);
 int get_canonic_origin(const char* o, char *co, int sz);
 int get_default_protocol_port(const char* scheme, size_t slen);
 
+///////////// HTTP ////////////////////
+
+void write_http_echo(ioa_socket_handle s);
+void handle_https(ioa_socket_handle s, ioa_network_buffer_handle nbh);
+
 ///////////////////////////////////////
 
 #ifdef __cplusplus
